// This example builds a simple HTML page with ON and OFF links to control max. 4 relays on
// the WEMOS D1 Mini enabled CANADUINO "Smarter Relays" module over WiFi, using a web browser. 
// The code needs modification for modules with less or more than 4 relays.
// 
// Parts of this code have been reused from the example on the ESP8266 Learning Website below. 
// http://www.esp8266learning.com/wemos-webserver-example.php
// 
// Volker Forster @ UNIVERSAL-SOLDER® 


//Including the libraries
#include <ESP8266WiFi.h>
#include "Wire.h"

// Setting up the HTML server and your own WiFi informaiton.
// The pre-set values are examples and need to be replaced by your actual information.
const char* ssid = "myWIFIname"; // Your WiFi name. It must be a 2.4GHz network.
const char* password = "myWIFIpassword"; // Your WiFi Password
WiFiServer server(80);
IPAddress ip(192, 168, 0, 222); // Replace this with the desired IP for your WEMOS D1 Mini
IPAddress gateway(192, 168, 0, 1); // Replace this with the standard gateway of your WiFi network
IPAddress subnet(255, 255, 255, 0); // Replace this with the subnet mask of your network

// Define 8-bit variable "address" for MCP23008 output port
byte  address = B00000000;

// The "void setup" will only run once after reset / powering up
void setup() {

// Setting up serial connection (I2C) to MCP23008 expander IC
// MCP23008 supports 8 different I2C addresses, to be set by the 3 jumpers A0, A1 and A2
// on your CANADUINO Smarter Relays module. The following table shows possible addresses.
// Where 'H' means jumper is open and 'L' means jumper is closed.
// 
// A0 A1 A2 Address
//  L  L  L 0x20
//  H  L  L 0x21
//  L  H  L 0x22
//  H  H  L 0x23
//  L  L  H 0x24
//  H  L  H 0x25
//  L  H  H 0x26
//  H  H  H 0x27
// 
// The default in this sketch is 0x27 (all jumpers open = HIGH)

  Wire.begin();
  Wire.beginTransmission(0x27);
  Wire.write(0x00);
  Wire.write(0x00);
  Wire.endTransmission();

// Setting up serial connection to the terminal for debugging  
  Serial.begin(9600);
  delay(10);

// Connecting to WiFi network
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.config(ip, gateway, subnet); 
  WiFi.begin(ssid, password);
  
// Showing dots during connecting... then status message after success
  while (WiFi.status() != WL_CONNECTED) {
    delay(250);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");
 
// Start the server
  server.begin();
  Serial.println("Server started");
 
// Print the IP address
  Serial.print("Use this URL : ");
  Serial.print("http://");
  Serial.print(WiFi.localIP());
  Serial.println("/"); 
}

// The "void loop" is where you put the perpetual running code
void loop() {

// Check if a client has connected
  WiFiClient client = server.available();
  if (!client) {
    return;
  }
 
// Wait until the client sends some data
  Serial.println("new client");
  while(!client.available()){
    delay(1);
  }
 
// Read the first line of the request
  String request = client.readStringUntil('\r');
  Serial.println(request);
  client.flush();
 
// Maintain relay 1 status or change it depending on the received string
  if (request.indexOf("/relay_1=ON") != -1) {
  Wire.beginTransmission(0x27);
  Wire.write(0x09);
  address = bitSet(address,0);
  Wire.write(address);
  Wire.endTransmission(); 
  delay(1000);
} 
  if (request.indexOf("/relay_1=OFF") != -1){
  Wire.beginTransmission(0x27);
  Wire.write(0x09);
  address = bitClear(address,0);
  Wire.write(address);
  Wire.endTransmission(); 
  delay(1000);
}
// Maintain relay 2 status or change it depending on the received string
  if (request.indexOf("/relay_2=ON") != -1) {
  Wire.beginTransmission(0x27);
  Wire.write(0x09);
  address = bitSet(address,1);
  Wire.write(address);
  Wire.endTransmission(); 
  delay(1000);
} 
  if (request.indexOf("/relay_2=OFF") != -1){
  Wire.beginTransmission(0x27);
  Wire.write(0x09);
  address = bitClear(address,1);
  Wire.write(address);
  Wire.endTransmission(); 
  delay(1000);
}
// Maintain relay 3 status or change it depending on the received string
  if (request.indexOf("/relay_3=ON") != -1) {
  Wire.beginTransmission(0x27);
  Wire.write(0x09);
  address = bitSet(address,2);
  Wire.write(address);
  Wire.endTransmission(); 
  delay(1000);
} 
  if (request.indexOf("/relay_3=OFF") != -1){
  Wire.beginTransmission(0x27);
  Wire.write(0x09);
  address = bitClear(address,2);
  Wire.write(address);
  Wire.endTransmission(); 
  delay(1000);
}
// Maintain relay 4 status or change it depending on the received string
  if (request.indexOf("/relay_4=ON") != -1) {
  Wire.beginTransmission(0x27);
  Wire.write(0x09);
  address = bitSet(address,3);
  Wire.write(address);
  Wire.endTransmission(); 
  delay(1000);
} 
  if (request.indexOf("/relay_4=OFF") != -1){
  Wire.beginTransmission(0x27);
  Wire.write(0x09);
  address = bitClear(address,3);
  Wire.write(address);
  Wire.endTransmission(); 
  delay(1000);
}

// Return the response, build the html page
  client.println("HTTP/1.1 200 OK");
  client.println("Content-Type: text/html");
  client.println(""); //  do not forget this one
  client.println("<!DOCTYPE HTML>");
  client.println("<html>");
  client.println("<br>");
  client.println("<a href=\"/relay_1=ON\">Click here to turn ON relay 1.</a> <br><br>");
  client.println("<a href=\"/relay_1=OFF\">Click here to turn OFF relay 1.</a><br><br><br>");
  client.println("<a href=\"/relay_2=ON\">Click here to turn ON relay 2.</a> <br><br>");
  client.println("<a href=\"/relay_2=OFF\">Click here to turn OFF relay 2.</a><br><br><br>");
  client.println("<a href=\"/relay_3=ON\">Click here to turn ON relay 3.</a> <br><br>");
  client.println("<a href=\"/relay_3=OFF\">Click here to turn OFF relay 3.</a><br><br><br>");
  client.println("<a href=\"/relay_4=ON\">Click here to turn ON relay 4.</a> <br><br>");
  client.println("<a href=\"/relay_4=OFF\">Click here to turn OFF relay 4.</a><br><br>");
  client.println("</html>");
 
  delay(1);
  Serial.println("Client disconnected");
  Serial.println("");
  Serial.print("address= ");
  Serial.println(address);
 
}//END
